# -*- coding: utf-8 -*-
# Déclarations pour l'interface d'admin Django
import datetime
import re

from django.contrib import admin
from django.shortcuts import get_object_or_404, render

# from django.contrib.admin.options import InlineModelAdmin
from djaa_list_filter.admin import AjaxAutocompleteListFilterModelAdmin

from grapes.bdd.admin_actions import (
    export_personne_csv,
    export_structure_csv,
    generate_mailto,
)
from grapes.bdd.admin_views import (
    abonnement_create,
    abonnement_paiement_create,
    adhesion_create,
    adhesion_paiement_create,
    adhesion_update,
    ps_get_urls,
)

from .forms import (
    LienAvecAPESForm,
    LienWebForm,
    PersonneForm,
    StructureForm,
    TerritoireForm,
)
from .models import (
    Abonnement,
    ActiviteAPES,
    Adhesion,
    Affiliation,
    AmbassadeurAPES,
    DomaineActivite,
    Fonction,
    LienAvecAPES,
    LienWeb,
    ParticulariteStructure,
    Personne,
    Relation,
    RendezVous,
    StatutJuridique,
    Structure,
    Territoire,
    TypeAmbassadeur,
    TypeLien,
    TypePartenaire,
)

ADMIN_ORDERING = [
    (
        'bdd',
        [
            # Contacts
            'Structure',
            'Relation',
            'Personne',
            # Infos des contacts souvent éditées
            'RendezVous',
            'Adhesion',
            'Abonnement',
            'LienAvecAPES',
            # Informations poussées des contacts,
            'LienWeb',
            # Listes de choix cachées aux éditeur·ice·s
            'ActiviteAPES',
            'AmbassadeurAPES',
            'Fonction',
            'DomaineActivite',
            'ParticulariteStructure',
            'StatutJuridique',
            'Territoire',
            'TypeAmbassadeur',
            'TypeLien',
            'TypePartenaire',
            # Informations supprimées à terme
            'Affiliation',
        ],
    ),
]


# Creating a sort function
def get_app_list(self, request):
    app_dict = self._build_app_dict(request)
    for app_name, object_list in ADMIN_ORDERING:
        app = app_dict[app_name]
        app['models'].sort(key=lambda x: object_list.index(x['object_name']))
        yield app


# Covering django.contrib.admin.AdminSite.get_app_lis
admin.AdminSite.get_app_list = get_app_list


def CustomTitledListFilter(title):
    class Wrapper(admin.FieldListFilter):
        def __new__(cls, *args, **kwargs):
            instance = admin.FieldListFilter.create(*args, **kwargs)
            instance.title = title
            return instance

    return Wrapper


class DateMAJListFilter(admin.SimpleListFilter):
    title = 'date de mise à jour'
    parameter_name = 'date-maj'

    def lookups(self, request, model_admin):
        return [
            ('1s', 'La dernière semaine'),
            ('1m', 'Le dernier mois (≤ 31 j)'),
            ('1t', 'Le dernier trimestre (0 à 92 j)'),
            ('1a', 'La dernière année (0 à 366 j)'),
            ('s1a', 'Depuis plus d’un an (≥ 366 j)'),
        ]

    def queryset(self, request, queryset):
        match self.value():
            case '1s':
                return queryset.filter(
                    date_maj__gte=datetime.date.today()
                    - datetime.timedelta(days=7),
                )
            case '1m':
                return queryset.filter(
                    date_maj__gte=datetime.date.today()
                    - datetime.timedelta(days=31),
                )
            case '1t':
                return queryset.filter(
                    date_maj__gte=datetime.date.today()
                    - datetime.timedelta(days=92),
                )
            case '1a':
                return queryset.filter(
                    date_maj__gte=datetime.date.today()
                    - datetime.timedelta(days=366),
                )
            case 's1a':
                return queryset.filter(
                    date_maj__lte=datetime.date.today()
                    - datetime.timedelta(days=366),
                )


class DateRDVListFilter(admin.SimpleListFilter):
    title = 'date de rendez-vous'
    parameter_name = 'date-rdv'

    def lookups(self, request, model_admin):
        return [
            ('dans-plus-d1an', "Dans plus d’un an (≥ 366 j)"),
            ('dans-l-annee', 'Dans l’année (≤ 366 j)'),
            ('dans-le-trimestre', 'Dans le trimestre (≤ 92 j)'),
            ('dans-le-mois', 'Dans le mois (≤ 31 j)'),
            ('dans-la-semaine', ' Dans la semaine (≤ 7 j)'),
            ('dans-le-futur', 'Dans le futur 🛸️'),
            ('en-cours', 'En cours 💬️'),
            ('dans-le-passe', 'Dans le passé 🏚️'),
            ('derniere-semaine', 'La dernière semaine (≤ 7 j)'),
            ('dernier-mois', 'Le dernier mois (≤ 31 j)'),
            ('dernier-trimestre', 'Le dernier trimestre (≤ 92 j)'),
            ('derniere-annee', 'La dernière année (≤ 366 j)'),
            ('1an-et-plus', 'Depuis plus d’un an (≥ 366 j)'),
        ]

    def queryset(self, request, queryset):
        match self.value():
            case 'dans-plus-d1an':
                return queryset.filter(
                    date_debut__gte=datetime.date.today()
                    + datetime.timedelta(days=366)
                )
            case 'dans-l-annee':
                return queryset.filter(
                    date_debut__gte=datetime.date.today(),
                    date_debut__lte=datetime.date.today()
                    + datetime.timedelta(days=366),
                )
            case 'dans-le-trimestre':
                return queryset.filter(
                    date_debut__gte=datetime.date.today(),
                    date_debut__lte=datetime.date.today()
                    + datetime.timedelta(days=92),
                )
            case 'dans-le-mois':
                return queryset.filter(
                    date_debut__gte=datetime.date.today(),
                    date_debut__lte=datetime.date.today()
                    + datetime.timedelta(days=31),
                )
            case 'dans-la-semaine':
                return queryset.filter(
                    date_debut__gte=datetime.date.today(),
                    date_debut__lte=datetime.date.today()
                    + datetime.timedelta(days=7),
                )
            case 'dans-le-futur':
                return queryset.filter(
                    date_debut__gt=datetime.date.today(),
                    date_fin__gt=datetime.date.today(),
                )
            case 'en-cours':
                return queryset.filter(
                    date_debut__lte=datetime.date.today(),
                    date_fin__gte=datetime.date.today(),
                )
            case 'dans-le-passe':
                return queryset.filter(
                    date_fin__lte=datetime.date.today(),
                    date_debut__lte=datetime.date.today(),
                )
            case 'derniere-semaine':
                return queryset.filter(
                    date_fin__lte=datetime.date.today(),
                    date_fin__gte=datetime.date.today()
                    - datetime.timedelta(days=7),
                )
            case 'dernier-mois':
                return queryset.filter(
                    date_fin__lte=datetime.date.today(),
                    date_fin__gte=datetime.date.today()
                    - datetime.timedelta(days=31),
                )
            case 'dernier-trimestre':
                return queryset.filter(
                    date_fin__lte=datetime.date.today(),
                    date_fin__gte=datetime.date.today()
                    - datetime.timedelta(days=92),
                )
            case 'derniere-annee':
                return queryset.filter(
                    date_fin__gte=datetime.date.today()
                    - datetime.timedelta(days=366),
                    date_fin__lte=datetime.date.today(),
                )
            case '1an-et-plus':
                return queryset.filter(
                    date_fin__lte=datetime.date.today()
                    - datetime.timedelta(days=366),
                )


# TODO: masquage par défaut dans les vues de listes
class MasqueListFilter(admin.SimpleListFilter):
    title = 'statut masqué'
    parameter_name = 'masque'

    def lookups(self, request, model_admin):
        return [
            ('1', 'Oui'),
            ('0', 'Non'),
        ]

    def queryset(self, request, queryset):
        match self.value():
            case '1':
                return queryset.filter(masque='1')
            case '0':
                return queryset.filter(masque='0')


class VisibilityOnIndexModelAdminMixin:
    def has_module_permission(self, request):
        """
        Récupère les permissions de la personne sur ce modèle et le masque
        de la page d'accueil si elle n'a que la permission 'view'.
        """
        model_perms = set()
        model_perm_re = re.compile(
            r'(?:{}\.)([a-z]+)(?:_{})'.format(
                self.opts.app_label, self.opts.model_name
            )
        )

        for perm in request.user.get_all_permissions():
            match = model_perm_re.match(perm)
            if match:
                model_perms.add(match.group(1))

        if not model_perms or model_perms == {'view'}:
            return False
        return True


class ButtonableModelAdmin(AjaxAutocompleteListFilterModelAdmin):
    """
    Add a 'buttons' context variable for display in change_form.html
    (but we'll use Django 1.1's admin views for the URL dispatching)
    http://www.djangosnippets.org/snippets/1016/
    """

    buttons = []

    def change_view(self, request, object_id, extra_context={}):
        context = extra_context.copy()
        context["buttons"] = self.buttons
        return super(ButtonableModelAdmin, self).change_view(
            request, object_id, extra_context=context
        )


class AmbassadeurPersonneInline(admin.TabularInline):
    model = AmbassadeurAPES
    extra = 0
    autocomplete_fields = [
        "type_ambassadeur",
    ]
    verbose_name = "Rôle"
    verbose_name_plural = "🦸🏿‍♀️ Rôles ambassadeur de la personne pour l’APES"


class AmbassadeurAPESAdmin(
    VisibilityOnIndexModelAdminMixin,
    AjaxAutocompleteListFilterModelAdmin,
):
    list_display = ["id", "personne", "type_ambassadeur"]
    list_display_links = ["id", "personne"]
    # ordering = ["structure", "personne"]
    # Par ordre inverse de la présentation dans l'admin
    autocomplete_list_filter = (
        'type_ambassadeur',
        'personne',
    )
    autocomplete_fields = [
        "personne",
        "type_ambassadeur",
    ]
    list_editable = [
        'type_ambassadeur',
    ]
    search_fields = [
        'personne__nom',
        'personne__prenom',
        'type_ambassadeur__libelle',
    ]


class LienAPESStructureInline(admin.StackedInline):
    model = LienAvecAPES
    extra = 0
    autocomplete_fields = ["personne_apes"]
    exclude = ["personne"]
    verbose_name = "Lien "
    verbose_name_plural = "🤝 Personne de l’APES en lien avec la structure"


class LienAPESPersonneInline(admin.TabularInline):
    model = LienAvecAPES
    extra = 0
    autocomplete_fields = ["personne_apes"]
    exclude = ["structure"]
    fk_name = "personne"
    verbose_name = "Lien "
    verbose_name_plural = "🤝 Personnes de l'APES en lien avec la personne"


class LienAvecAPESAdmin(
    VisibilityOnIndexModelAdminMixin,
    AjaxAutocompleteListFilterModelAdmin,
):
    list_display = ["id", "structure", "personne", "personne_apes"]
    list_display_links = ["id", "structure", "personne"]
    ordering = ["structure", "personne"]
    # Par ordre inverse de la présentation dans l'admin
    autocomplete_list_filter = ("personne_apes", "personne", "structure")
    autocomplete_fields = ["structure", "personne", "personne_apes"]
    list_editable = ["personne_apes"]
    search_fields = [
        "personne__nom",
        "personne__prenom",
        "personne_apes__nom",
        "personne_apes__prenom",
        "structure__nom",
    ]
    form = LienAvecAPESForm


class RelationPersonneInline(admin.StackedInline):
    model = Relation
    extra = 0
    autocomplete_fields = [
        "fonction",
        "personne",
        "structure",
    ]
    verbose_name_plural = "↔️ Relations de la personne avec des structures"


class RelationStructureInline(admin.StackedInline):
    model = Relation
    extra = 0
    autocomplete_fields = [
        "fonction",
        "personne",
        "structure",
    ]
    verbose_name_plural = "↔️ Relations de la structure avec des personnes"


class RelationAdmin(
    VisibilityOnIndexModelAdminMixin,
    AjaxAutocompleteListFilterModelAdmin,
):
    list_display = ["id", "personne", "fonction", "structure"]
    list_display_links = ["structure", "personne"]
    ordering = ["structure", "fonction", "personne"]
    # Par ordre inverse de la présentation dans l'admin
    autocomplete_list_filter = (
        'structure',
        'fonction',
        'personne',
    )
    autocomplete_fields = ["fonction"]
    list_editable = ["fonction"]
    search_fields = [
        "personne__nom",
        "personne__prenom",
        "fonction__libelle",
        "structure__nom",
    ]


class LienWebStructureInline(admin.StackedInline):
    model = LienWeb
    extra = 0
    autocomplete_fields = [
        "type_lien",
    ]
    exclude = ["personne"]
    verbose_name_plural = "🌐 Liens web rattachés à la structure"


class LienWebPersonneInline(admin.StackedInline):
    model = LienWeb
    extra = 0
    autocomplete_fields = [
        "type_lien",
    ]
    exclude = ["structure"]
    verbose_name_plural = "🌐 Liens web rattachés à la personne"


class LienWebAdmin(AjaxAutocompleteListFilterModelAdmin):
    list_display = ["id", "type_lien", "url"]
    list_display_links = ["id", "type_lien"]
    ordering = ["type_lien", "url"]
    # Par ordre inverse de la présentation dans l'admin
    autocomplete_list_filter = [
        'type_lien',
        'personne',
        'structure',
    ]
    autocomplete_fields = ["personne", "structure", "type_lien"]
    list_editable = ["url"]
    search_fields = [
        "personne__nom",
        "personne__prenom",
        "structure__nom",
        "type_lien__libelle",
        "url",
    ]

    form = LienWebForm


# class RendezVousPersonneInline(admin.TabularInline):
#     model = RendezVous
#     extra = 0
#     autocomplete_fields = [
#         "structures_presentes",
#     ]
#     exclude = ["structure"]
#     fields=["personnes_apes"]
#     list_select_related=["personnes_apes"]

# class RendezVousStructureInline(admin.TabularInline):
#     model = RendezVous.autres_personnes_presentes.through
#     extra = 0
#     autocomplete_fields = [
#         "structures_presentes",
#     ]
#     exclude = ["structure"]


class RendezVousAdmin(
    VisibilityOnIndexModelAdminMixin,
    AjaxAutocompleteListFilterModelAdmin,
):
    @admin.display(description='Personnes APES')
    def get_personnes_apes(self, obj):
        if obj.personnes_apes.all():
            return list(obj.personnes_apes.all())
        else:
            return '❌ Aucune personne de l’APES'

    @admin.display(description='Autres personnes')
    def get_autres_personnes(self, obj):
        if obj.autres_personnes_presentes.all():
            return list(obj.autres_personnes_presentes.all())
        else:
            return '❌ Aucune autre personne'

    @admin.display(description='Structures')
    def get_structures(self, obj):
        if obj.structures_presentes.all():
            return list(
                obj.structures_presentes.all().values_list('nom', flat=True)
            )
        else:
            return 'Aucune structure'

    list_display = [
        "id",
        "date_debut",
        "date_fin",
        "get_personnes_apes",
        "get_autres_personnes",
        "get_structures",
    ]
    list_display_links = [
        "id",
        "date_debut",
        "date_fin",
        "get_personnes_apes",
        "get_autres_personnes",
        "get_structures",
    ]
    ordering = [
        "date_debut",
        "date_fin",
    ]
    # Par ordre inverse de la présentation dans l'admin
    autocomplete_list_filter = (
        'autres_personnes_presentes',
        'personnes_apes',
        'structures_presentes',
    )
    list_filter = [DateRDVListFilter]
    autocomplete_fields = [
        "personnes_apes",
        "autres_personnes_presentes",
        "structures_presentes",
    ]
    search_fields = [
        "personnes_apes__nom",
        "personnes_apes__prenom",
        "autres_personnes_presentes__nom",
        "autres_personnes_presentes__prenom",
        "structures_presentes__nom",
    ]
    fieldsets = (
        ("🗓️  Les dates", {"fields": (("date_debut", "date_fin"),)}),
        (
            "👤 🏢  Les personnes et structures",
            {
                "fields": (
                    (
                        "personnes_apes",
                        "autres_personnes_presentes",
                        "structures_presentes",
                    ),
                )
            },
        ),
        (
            "🗒️ Les détails du rendez-vous",
            {"fields": (("commentaires",), ("piece_jointe",))},
        ),
    )


class PersonneAdmin(VisibilityOnIndexModelAdminMixin, ButtonableModelAdmin):
    @admin.display(description='activité(s) APES')
    def get_activites_apes(self, obj):
        if obj.activites_apes.all():
            return list(
                obj.activites_apes.all().values_list('libelle', flat=True)
            )
        else:
            return 'Aucune activité APES'

    @admin.display(description='domaine(s) d’activité')
    def get_domaines_activite(self, obj):
        if obj.domaines_activite.all():
            return list(
                obj.domaines_activite.all().values_list('libelle', flat=True)
            )
        else:
            return 'Aucun domaine d’activité'

    @admin.display(description='territoire(s)')
    def get_territoires(self, obj):
        if obj.territoires.all():
            return list(
                obj.territoires.all().values_list('libelle', flat=True)
            )
        else:
            return 'Aucun territoire'

    list_display = [
        "id",
        "nom",
        "prenom",
        "date_maj",
        "echeance_adhesion",
        "get_territoires",
        "get_activites_apes",
        "get_domaines_activite",
    ]
    list_display_links = [
        "id",
        "nom",
        "prenom",
        "date_maj",
        "echeance_adhesion",
        "get_territoires",
        "get_activites_apes",
        "get_domaines_activite",
    ]
    # Par ordre inverse de la présentation dans l'admin
    autocomplete_list_filter = (
        "structure__relations",
        "domaines_activite",
        "type_partenaire",
        "activites_apes",
        "territoires",
    )
    list_filter = [
        DateMAJListFilter,
        ('adherent', CustomTitledListFilter('statut adhérent')),
        ('abonne', CustomTitledListFilter('statut abonné')),
        MasqueListFilter,
        ('a_supprimer', CustomTitledListFilter('statut à supprimer')),
    ]
    search_fields = [
        "id",
        "nom",
        "prenom",
        "territoires__libelle",
        "activites_apes__libelle",
        "domaines_activite__libelle",
    ]
    autocomplete_fields = [
        "ambassadeur_apes",
        "activites_apes",
        "domaines_activite",
        "territoires",
        "type_partenaire",
    ]
    readonly_fields = ["date_maj"]
    actions = [generate_mailto, export_personne_csv]
    list_per_page = 300
    fieldsets = (
        (
            "🗒️ L'état de la personne",
            {
                "fields": (
                    (
                        "date_maj",
                        "adherent",
                        "abonne",
                        "masque",
                        "a_supprimer",
                    ),
                )
            },
        ),
        (
            "🗺️ Les coordonnées de la personne",
            {
                "fields": (
                    ("nom", "prenom"),
                    "adresse",
                    ("code_postal", "ville"),
                    ("telephone1", "telephone2"),
                    "email",
                    # ("sol", "journaliste"),
                )
            },
        ),
        (
            "🔘 Son périmètre",
            {"fields": (("domaines_activite", "territoires"),)},
        ),
        (
            "🔵 Ses spécificités APES",
            {
                "fields": (
                    (
                        "activites_apes",
                        "type_partenaire",
                        "referent_apes",
                    ),
                )
            },
        ),
    )
    inlines = (
        AmbassadeurPersonneInline,
        LienAPESPersonneInline,
        RelationPersonneInline,
        LienWebPersonneInline,
    )
    get_urls = ps_get_urls
    abonnement_create = abonnement_create
    adhesion_create = adhesion_create
    adhesion_update = adhesion_update
    abonnement_paiement_create = abonnement_paiement_create
    adhesion_paiement_create = adhesion_paiement_create

    form = PersonneForm

    def affiliation_detail(self, request, object_id):
        """
        Create the affiliation if necessary and display it
        """
        object = get_object_or_404(self.model, pk=object_id)
        try:
            affiliation = Affiliation.objects.get(personne=object)
        except Affiliation.DoesNotExist:
            affiliation = Affiliation(personne=object)
            affiliation.save()
        return render(
            request,
            "bdd/affiliation_detail.html",
            {
                "title": self.affiliation_detail.short_description,
                "current_app": self.admin_site.name,  # useful?
                "object": affiliation,
            },
        )

    affiliation_detail.short_description = "Affiliations"

    buttons = ((affiliation_detail.short_description, "affiliation/"),)


class StructureAdmin(VisibilityOnIndexModelAdminMixin, ButtonableModelAdmin):
    @admin.display(description='activité(s) APES')
    def get_activites_apes(self, obj):
        if obj.activites_apes.all():
            return list(
                obj.activites_apes.all().values_list('libelle', flat=True)
            )
        else:
            return 'Aucune activité APES'

    @admin.display(description='domaine(s) d’activité')
    def get_domaines_activite(self, obj):
        if obj.domaines_activite.all():
            return list(
                obj.domaines_activite.all().values_list('libelle', flat=True)
            )
        else:
            return '❌ Aucun domaine d’activité'

    @admin.display(description='territoire(s)')
    def get_territoires(self, obj):
        if obj.territoires.all():
            return list(
                obj.territoires.all().values_list('libelle', flat=True)
            )
        else:
            return '❌ Aucun territoire'

    @admin.display(description='abonnement(s)')
    def get_abonnements(self, obj):
        if Affiliation.objects.filter(structure=obj).exists():
            A = Affiliation.objects.filter(structure=obj).first()
            if Abonnement.objects.filter(affiliation=A).exists():
                Ab = Abonnement.objects.filter(affiliation=A).first()
                return Ab.nb_exemplaires
        return 'Aucun'

    list_display = (
        "id",
        "nom",
        "date_maj",
        "adherent",
        "echeance_adhesion",
        "abonne",
        "get_abonnements",
        "get_territoires",
        "get_activites_apes",
        "get_domaines_activite",
    )
    list_display_links = (
        "echeance_adhesion",
        "nom",
    )
    # Par ordre inverse de la présentation dans l'admin
    autocomplete_list_filter = (
        "relations",
        "statut_juridique",
        "domaines_activite",
        "activites_apes",
        "territoires",
        #       "demarche_progres",
        #       "marche_citoyen",
        #       "collectifs_acteurs",
        #       "sol",
        #       "institutionnel",
        #       "journaliste",
    )
    list_filter = (
        DateMAJListFilter,
        ('adherent', CustomTitledListFilter('statut adhérent')),
        ('abonne', CustomTitledListFilter('statut abonné')),
        MasqueListFilter,
        ('a_supprimer', CustomTitledListFilter('statut à supprimer')),
    )
    list_per_page = 100
    search_fields = [
        "id",
        "activites_apes__libelle",
        "nom",
        "domaines_activite__libelle",
        "territoires__libelle",
        "ville",
    ]
    autocomplete_fields = [
        'activites_apes',
        'domaines_activite',
        'particularites_structure',
        'statut_juridique',
        'territoires',
        'type_partenaire',
    ]
    inlines = [
        LienAPESStructureInline,
        RelationStructureInline,
        LienWebStructureInline,
    ]
    readonly_fields = [
        'date_maj',
        'domaines_activite_txt',
    ]
    actions = (generate_mailto, export_structure_csv)
    get_urls = ps_get_urls
    abonnement_create = abonnement_create
    adhesion_create = adhesion_create
    adhesion_update = adhesion_update
    abonnement_paiement_create = abonnement_paiement_create
    adhesion_paiement_create = adhesion_paiement_create

    form = StructureForm

    fieldsets = (
        (
            "🗒️ L'état de la structure",
            {
                "fields": (
                    (
                        "date_maj",
                        "adherent",
                        "abonne",
                        "masque",
                        "a_supprimer",
                    ),
                )
            },
        ),
        (
            "🗺️ Les coordonnées de la structure",
            {
                "fields": (
                    "nom",
                    "adresse",
                    ("code_postal", "ville"),
                    ("telephone1", "telephone2"),
                    ("email", "site"),
                )
            },
        ),
        (
            "🔘 Son périmètre",
            {
                "fields": (
                    ("territoires", "domaines_activite"),
                    "domaines_activite_txt",
                    # "nb_salaries",
                    # "mission",
                    # (
                    # "demarche_progres",
                    # "demarche_progres_environnement",
                    # "demarche_progres_emploi",
                    # ),
                    # "marche_citoyen",
                    # "collectifs_acteurs",
                    # "sol",
                    # "institutionnel",
                    # "journaliste",
                    # "autres_contacts",
                ),
            },
        ),
        (
            "⚖️ Ses spécificités juridiques",
            {"fields": (("statut_juridique", "particularites_structure"),)},
        ),
        (
            "🔵 Ses spécificités APES",
            {"fields": (("activites_apes", "type_partenaire"),)},
        ),
    )

    def affiliation_detail(self, request, object_id):
        """
        s/personne/structure/ Personne.affiliation_detail
        """
        object = get_object_or_404(self.model, pk=object_id)
        try:
            affiliation = Affiliation.objects.get(structure=object)
        except Affiliation.DoesNotExist:
            affiliation = Affiliation(structure=object)
            affiliation.save()
        return render(
            request,
            "bdd/affiliation_detail.html",
            {
                "title": self.affiliation_detail.short_description,
                "current_app": self.admin_site.name,  # useful?
                "object": affiliation,
            },
        )

    affiliation_detail.short_description = "Affiliations"

    buttons = ((affiliation_detail.short_description, "affiliation/"),)


# class CompetenceAdmin(admin.ModelAdmin):
#    list_display = ['id', 'nom', 'prenom']
#    list_display_links = ['id']


# Won't work because the OneToOneField isn't is Paiement
# class ModePaiementAbonnementInline(admin.TabularInline):
#    model = ModePaiementAbonnement
class AbonnementAdmin(
    VisibilityOnIndexModelAdminMixin, AjaxAutocompleteListFilterModelAdmin
):
    list_display = ["id", "date", "nb_exemplaires", "nom_associe"]
    list_display_links = ["id", "date"]
    list_filter = ["date", "nb_exemplaires"]
    autocomplete_list_filter = ["affiliation"]
    date_hierarchy = "date"
    search_fields = ["nom_associe"]


#    inlines = [ModePaiementAbonnementInline]


# Won't work because the OneToOneField isn't is Paiement
# class ModePaiementAdhesionInline(admin.TabularInline):
#    model = ModePaiementAdhesion


class DateAdhesionListFilter(admin.SimpleListFilter):
    title = 'date de la dernière adhésion'
    parameter_name = 'date-adhesion'

    def lookups(self, request, model_admin):
        return [
            ('1s', 'La dernière semaine'),
            ('1m', 'Le dernier mois (≤ 31 j)'),
            ('1t', 'Le dernier trimestre (0 à 92 j)'),
            ('1a', 'La dernière année (0 à 366 j)'),
            ('a-1', 'Entre 1 et 2 ans avant (366 à 731 j)'),
            ('a-2', 'Entre 2 et 3 ans avant (731 à 1096 j)'),
            ('3a', 'Les 3 dernières années (0 à 1096 j)'),
            ('s3a', 'Il y’a plus de 3 ans (≥ 1097 j)'),
        ]

    def queryset(self, request, queryset):
        match self.value():
            case '1s':
                return queryset.filter(
                    date__gte=datetime.date.today()
                    - datetime.timedelta(days=7),
                )
            case '1m':
                return queryset.filter(
                    date__gte=datetime.date.today()
                    - datetime.timedelta(days=31),
                )
            case '1t':
                return queryset.filter(
                    date__gte=datetime.date.today()
                    - datetime.timedelta(days=92),
                )
            case '1a':
                return queryset.filter(
                    date__gte=datetime.date.today()
                    - datetime.timedelta(days=366),
                )
            case 'a-1':
                return queryset.filter(
                    date__lte=datetime.date.today()
                    - datetime.timedelta(days=366),
                    date__gte=datetime.date.today()
                    - datetime.timedelta(days=731),
                )
            case 'a-2':
                return queryset.filter(
                    date__lte=datetime.date.today()
                    - datetime.timedelta(days=731),
                    date__gte=datetime.date.today()
                    - datetime.timedelta(days=1096),
                )
            case '3a':
                return queryset.filter(
                    date__lte=datetime.date.today(),
                    date__gte=datetime.date.today()
                    - datetime.timedelta(days=1096),
                )
            case 's3a':
                return queryset.filter(
                    date__lte=datetime.date.today()
                    - datetime.timedelta(days=1097),
                )


class AdhesionAdmin(
    VisibilityOnIndexModelAdminMixin,
    AjaxAutocompleteListFilterModelAdmin,
):
    list_display = ["id", "date", "college", "nom_associe"]
    list_display_links = ["id", "date"]
    list_filter = [
        DateAdhesionListFilter,
        "college",
        "commission",
        "collectif_acteur_txt",
    ]
    date_hierarchy = "date"

    autocomplete_list_filter = ("affiliation",)

    readonly_fields = ["affiliation"]
    search_fields = ["nom_associe"]


class ModePaiementAbonnementAdmin(admin.ModelAdmin):
    list_display = [
        "id",
        "montant",
        "type",
        "date_reglement",
        "date_facturation",
    ]
    list_display_links = ["id"]
    list_filter = ["montant", "type", "date_reglement", "date_facturation"]


class ModePaiementAdhesionAdmin(admin.ModelAdmin):
    list_display = [
        "id",
        "montant",
        "type",
        "date_reglement",
        "date_facturation",
    ]
    list_display_links = ["id"]
    list_filter = ["montant", "type", "date_reglement", "date_facturation"]


class AffiliationAdmin(VisibilityOnIndexModelAdminMixin, admin.ModelAdmin):
    list_display = [
        "id",
        "etat",
        "date_maj",
        "nom",
        "prenom",
        "nom_precedent",
        "montant_total",
    ]
    list_display_links = ["id", "etat", "date_maj"]
    list_filter = [
        "date_maj",
        "etat",
        MasqueListFilter,
    ]
    readonly_fields = ["personne", "structure"]
    search_fields = [
        "id",
        "adresse_complete",
        "email",
        "nom_precedent",
        "structure__nom",
        "personne__nom",
        "personne__prenom",
    ]


class LibelleAdmin(VisibilityOnIndexModelAdminMixin, admin.ModelAdmin):
    ordering = ["libelle", "id"]
    list_display = ["id", "libelle", "slug"]
    list_display_links = ["id", "slug"]
    list_editable = ["libelle"]
    search_fields = ["libelle"]
    prepopulated_fields = {"slug": ("libelle",)}


class TerritoireAdmin(
    VisibilityOnIndexModelAdminMixin, AjaxAutocompleteListFilterModelAdmin
):
    list_display = ["id", "libelle", "slug", "parent"]
    list_display_links = ["id", "slug"]
    list_editable = ["libelle", "parent"]
    search_fields = ["libelle"]
    autocomplete_fields = ["parent"]
    autocomplete_list_filter = ["parent"]
    prepopulated_fields = {"slug": ("libelle",)}
    form = TerritoireForm


def register_bdd_models(admin_site):
    # admin_site.register(Contact)
    admin_site.register(Abonnement, AbonnementAdmin)
    admin_site.register(ActiviteAPES, LibelleAdmin)
    admin_site.register(Adhesion, AdhesionAdmin)
    admin_site.register(Affiliation, AffiliationAdmin)
    admin_site.register(AmbassadeurAPES, AmbassadeurAPESAdmin)
    admin_site.register(DomaineActivite, LibelleAdmin)
    admin_site.register(Fonction, LibelleAdmin)
    admin_site.register(LienWeb, LienWebAdmin)
    admin_site.register(LienAvecAPES, LienAvecAPESAdmin)
    # admin_site.register(ModePaiementAbonnement, ModePaiementAbonnementAdmin)
    # admin_site.register(ModePaiementAdhesion, ModePaiementAdhesionAdmin)
    admin_site.register(ParticulariteStructure, LibelleAdmin)
    admin_site.register(Personne, PersonneAdmin)
    admin_site.register(Relation, RelationAdmin)
    admin_site.register(RendezVous, RendezVousAdmin)
    admin_site.register(StatutJuridique, LibelleAdmin)
    admin_site.register(Structure, StructureAdmin)
    admin_site.register(Territoire, TerritoireAdmin)
    admin_site.register(TypeAmbassadeur, LibelleAdmin)
    admin_site.register(TypeLien, LibelleAdmin)
    admin_site.register(TypePartenaire, LibelleAdmin)


register_bdd_models(admin.site)
