# -*- coding: utf-8 -*-
# Modèles de l’application

import datetime

from django.db import models
from django.utils.formats import localize
from django.utils.html import format_html
from django.utils.text import slugify
from django.utils.timezone import now

# Create your models here.


def echeance_abonnement(self, htmlized=True):
    date_derniere_facture = self.affiliation_set.aggregate(
        models.Max("abonnement__date")
    )["abonnement__date__max"]
    if date_derniere_facture is None:
        return None
    else:
        ret = localize(date_derniere_facture)
        if date_derniere_facture < (
            datetime.date.today() - datetime.timedelta(365)
        ):
            if htmlized:
                ret = format_html("<em>{}</em>", ret)
            else:
                ret = "%s" % ret
        return ret


echeance_abonnement.admin_order_field = "affiliation__abonnement__date"
echeance_abonnement.allow_tags = True
echeance_abonnement.short_description = "Date Abonnement"


def echeance_adhesion(self, htmlized=True):
    date_derniere_facture = self.affiliation_set.aggregate(
        models.Max("adhesion__date")
    )["adhesion__date__max"]
    if date_derniere_facture is None:
        return None
    else:
        ret = localize(date_derniere_facture)
        if date_derniere_facture < (
            datetime.date.today() - datetime.timedelta(365)
        ):
            if htmlized:
                ret = format_html("<em>{}</em>", ret)
            else:
                ret = "%s" % ret
        return ret


echeance_adhesion.admin_order_field = "affiliation__adhesion__date"
echeance_adhesion.allow_tags = True
echeance_adhesion.short_description = "Date Adhésion"


def get_college(contact):
    try:
        last_af = contact.affiliation_set.latest("adhesion__date")
    except Affiliation.DoesNotExist:
        last_af = None
    try:
        college = last_af and last_af.adhesion and last_af.adhesion.college
    except Adhesion.DoesNotExist:
        college = None
    return college or ""


class Personne(models.Model):  # (Contact):
    class Meta:
        ordering = ["nom", "prenom"]

    # contact_ptr = models.IntegerField()
    nom = models.CharField(max_length=255, blank=True)
    prenom = models.CharField("prénom", max_length=255, blank=True)

    adresse = models.CharField(max_length=255, blank=True)
    code_postal = models.CharField(max_length=255, blank=True)
    ville = models.CharField(max_length=255, blank=True)
    telephone1 = models.CharField("téléphone 1", max_length=255, blank=True)
    telephone2 = models.CharField("téléphone 2", max_length=255, blank=True)
    email = models.EmailField(blank=True)
    type_partenaire = models.ManyToManyField("TypePartenaire", blank=True)
    abonne = models.BooleanField("abonné", default=False)
    adherent = models.BooleanField("adhérent", default=False)
    sol = models.BooleanField("SOL", default=False)
    journaliste = models.BooleanField("journaliste", default=False)
    activites_apes = models.ManyToManyField(
        "ActiviteAPES",
        blank=True,
        help_text=(
            "ℹ️ Activité(s) de <em>l'APES</em> dans lesquelles la personne est"
            "active."
        ),
        verbose_name="activité de l’APES",
    )
    ambassadeur_apes = models.ManyToManyField(
        "TypeAmbassadeur",
        blank=True,
        help_text="ℹ️ Relais local, ambassadeur·ice <abbr>PLUSS</abbr>…",
        through="AmbassadeurAPES",
        verbose_name="ambassadeur ou ambassadrice APES",
    )
    referent_apes = models.BooleanField(
        "Référent APES",
        default=False,
        help_text="ℹ️ La personne représente-t-iel l’<em>APES</em> ?",
    )
    domaines_activite = models.ManyToManyField(
        "DomaineActivite",
        blank=True,
        help_text=(
            "Domaine(s) d’activité dans le(s)quel(s) la personne exerce."
        ),
        verbose_name="domaine d’activité",
    )
    territoires = models.ManyToManyField(
        "Territoire",
        blank=True,
        help_text=(
            "Territoire(s) dans lequel la personne est active. Peut être"
            " différent de sa localisation."
        ),
    )
    adresse_complete = models.TextField(editable=False, blank=True)
    echeance_adhesion = echeance_adhesion
    echeance_abonnement = echeance_abonnement
    relations = models.ManyToManyField(
        "Structure", through="Relation", verbose_name="relations"
    )
    liens_avec_apes = models.ManyToManyField(
        "Personne", through="LienAvecAPES", verbose_name="Liens avec l'APES"
    )
    masque = models.BooleanField(
        "Masqué",
        default=False,
        help_text=(
            "ℹ️ Permet de masquer la personne via un filtre de liste et dans "
            "certains champs. ⚠️ La personne n’est plus sélectionnable dans "
            "certains outils de la base."
        ),
    )
    a_supprimer = models.BooleanField(
        "à supprimer",
        default=False,
        help_text=(
            "⚠️ Note la fiche comme à supprimer par les administrateur·ice·s"
        ),
    )
    date_maj = models.DateField("date de mise à jour", auto_now=True)

    def nom_complet(self):
        return "%s %s" % (self.prenom, self.nom)

    # TODO : rajouter la mise à jour de l'adresse de l'affiliation si un jour,
    # on a besoin de lister tout ou une partie des adresses des affiliations.
    def save(self, force_insert=False, force_update=False):
        self.adresse_complete = "%s %s\n%s\n%s %s" % (
            self.prenom,
            self.nom.upper(),
            self.adresse,
            self.code_postal,
            self.ville,
        )
        super(Personne, self).save(force_insert, force_update)

    def get_college(self):
        return get_college(self)

    def __str__(self):
        return f"{self.prenom} {self.nom} (P. №{self.pk})"


class Structure(models.Model):  # (Contact):
    class Meta:
        ordering = ["nom"]

    # contact_ptr = models.IntegerField()
    nom = models.CharField(max_length=255, null=True)

    adresse = models.CharField(max_length=255, blank=True)
    code_postal = models.CharField(max_length=255, blank=True)
    ville = models.CharField(max_length=255, blank=True)
    telephone1 = models.CharField("téléphone 1", max_length=255, blank=True)
    telephone2 = models.CharField("téléphone 2", max_length=255, blank=True)
    email = models.EmailField(blank=True)
    site = models.CharField(max_length=255, blank=True)
    type_partenaire = models.ManyToManyField(
        "TypePartenaire",
        blank=True,
        help_text="Acteur·ice, porteur·euse de projet, élu·e, technicien·ne…",
    )
    abonne = models.BooleanField("abonné", default=False)
    adherent = models.BooleanField(
        "adhérent",
        default=False,
    )
    activites_apes = models.ManyToManyField(
        "ActiviteAPES",
        blank=True,
        help_text=(
            "Activité(s) de l'APES dans lesquelles la structure est active."
        ),
        verbose_name="activité(s) de l’APES",
    )
    territoires = models.ManyToManyField(
        "Territoire",
        blank=True,
        help_text=(
            "Territoire(s) dans lequel la structure est active. Peut être"
            " différent de sa localisation."
        ),
        verbose_name="territoire(s)",
    )
    # adresse_complete = models.TextField(editable=False, blank=True)

    domaines_activite_txt = models.TextField(
        blank=True,
        help_text="⚠️ Champs déprécié. Était en saisie libre.",
        verbose_name="Ancien champs domaines d’activité",
    )

    domaines_activite = models.ManyToManyField(
        "DomaineActivite",
        blank=True,
        help_text=(
            "Domaine(s) d’activité dans le(s)quel(s) la structure exerce."
        ),
        verbose_name="domaine d’activité",
    )

    statut_juridique = models.ForeignKey(
        "StatutJuridique",
        blank=True,
        null=True,
        on_delete=models.PROTECT,
        help_text=("Association loi 1901, SA, SARL…"),
    )

    particularites_structure = models.ManyToManyField(
        "ParticulariteStructure",
        blank=True,
        help_text=(
            "Entreprise solidaire d'utilité sociale (ESUS), coopérative"
            " d’actvité et d’emploi (CAE)"
        ),
        verbose_name="Particularité de la structure",
    )
    # date_creation = models.DateField(
    #     "date de création", blank=True, null=True
    # )
    nb_salaries = models.IntegerField(
        "nombre de salariés", blank=True, null=True
    )
    mission = models.TextField(blank=True)
    # horaires_ouverture = models.TextField(blank=True)
    demarche_progres = models.BooleanField(
        "démarche progrès",
        blank=True,
        default=False,
        null=True,
    )
    demarche_progres_environnement = models.BooleanField(
        "si oui Environnement",
        blank=True,
        default=False,
        null=True,
    )
    demarche_progres_emploi = models.BooleanField(
        "et ou Emploi",
        blank=True,
        default=False,
        null=True,
    )
    marche_citoyen = models.BooleanField(
        "BDIS Pro",
        blank=True,
        default=False,
    )

    collectifs_acteurs = models.BooleanField(
        "Collectif acteurs",
        blank=True,
        default=False,
    )
    sol = models.BooleanField(
        "SOL",
        blank=True,
        default=False,
    )
    institutionnel = models.BooleanField(
        "Institutionnel",
        blank=True,
        default=False,
    )
    journaliste = models.BooleanField(
        "Journaliste",
        blank=True,
        default=False,
    )
    relations = models.ManyToManyField("Personne", through="Relation")

    adresse_contact_etiquette = models.TextField(blank=True, editable=False)
    autres_contacts = models.CharField(
        "Contact etiquette", max_length=255, blank=True
    )
    echeance_adhesion = echeance_adhesion
    echeance_abonnement = echeance_abonnement

    masque = models.BooleanField(
        "masqué",
        default=False,
        help_text=(
            "ℹ️ Permet de masquer la structure via un filtre de liste et dans "
            "certains champs. ⚠️ La structure n’est plus sélectionnable dans "
            "certains outils de la base."
        ),
    )

    a_supprimer = models.BooleanField(
        "à supprimer",
        default=False,
        help_text=(
            "⚠️ Note la fiche comme à supprimer par les administrateur·ice·s"
        ),
    )
    date_maj = models.DateField("Date de mise à jour", auto_now=True)

    def nom_complet(self):
        return self.nom

    #    def save(self, force_insert=False, force_update=False):
    #        if self.personne1:
    #            self.nom_prenom_personne1 = "%s %s" % (
    #                self.personne1.prenom, self.personne1.nom.upper())
    #        else:
    #            self.nom_prenom_personne1 = ""
    #        self.adresse_complete = "%s\n%s\n%s\n%s %s" % (
    #            self.nom, self.nom_prenom_personne1, self.adresse,
    #            self.code_postal, self.ville)
    #        super(Structure, self).save(force_insert, force_update)

    def __str__(self):
        return f"{self.nom} (S. №{self.pk})"

    def get_college(self):
        return get_college(self)

    def save(self, force_insert=False, force_update=False):
        self.adresse_contact_etiquette = "%s\n%s\n%s\n%s %s" % (
            self.nom,
            self.autres_contacts,
            self.adresse,
            self.code_postal,
            self.ville,
        )
        super(Structure, self).save(force_insert, force_update)


class Abonnement(models.Model):
    class Meta:
        ordering = ["date"]

    affiliation = models.OneToOneField("Affiliation", on_delete=models.CASCADE)
    paiement = models.OneToOneField(
        "ModePaiementAbonnement",
        blank=True,
        null=True,
        on_delete=models.PROTECT,
    )
    nb_exemplaires = models.IntegerField(blank=True, default=1, null=True)
    adresse_differente = models.CharField(
        max_length=255, editable=False, blank=True
    )
    # TODO: remove, unused except for once and it's not justified
    date = models.DateField(default=datetime.date.today, blank=True, null=True)
    nom_associe = models.CharField(
        "nom associé (copie)", max_length=255, editable=False, blank=True
    )

    def save(self, force_insert=False, force_update=False):
        if self.affiliation.structure is not None:
            self.nom_associe = self.affiliation.structure.nom
            Structure.objects.filter(pk=self.affiliation.structure.id).update(
                abonne=True
            )
        elif self.affiliation.personne is not None:
            self.nom_associe = "%s %s" % (
                self.affiliation.personne.prenom,
                self.affiliation.personne.nom,
            )
            (
                Personne.objects.filter(
                    pk=self.affiliation.personne.id
                ).update(abonne=True)
            )
        super(Abonnement, self).save(force_insert, force_update)

    def nom(self):
        return self.affiliation.nom()

    def prenom(self):
        return self.affiliation.prenom()

    prenom.short_description = "prénom"

    def __str__(self):
        if self.affiliation.structure is not None:
            return "Abonnement %d : %s de %s" % (
                self.pk,
                localize(self.date),
                self.affiliation.structure.nom,
            )
        elif self.affiliation.personne is not None:
            return "Abonnement %d : %s de %s %s" % (
                self.pk,
                localize(self.date),
                self.affiliation.personne.prenom,
                self.affiliation.personne.nom,
            )
        return "Abonnement %d : (orphelin)" % self.pk


class Adhesion(models.Model):
    class Meta:
        ordering = ["date"]
        verbose_name = "Adhésion"

    affiliation = models.OneToOneField(
        "Affiliation", blank=True, null=True, on_delete=models.CASCADE
    )
    paiement = models.OneToOneField(
        "ModePaiementAdhesion", blank=True, null=True, on_delete=models.PROTECT
    )
    COLLEGE_CHOIX = [
        ["A", "A"],
        ["B", "B"],
    ]
    college = models.CharField(max_length=1, choices=COLLEGE_CHOIX, blank=True)
    # fonction_adherent = models.CharField("fonction de l'adhérent",
    #  max_length=255, blank=True)  # TODO: choices
    commission = models.CharField(max_length=255, editable=False, blank=True)
    # => unused, should be choices?
    collectif_acteur_txt = models.TextField(editable=False, blank=True)
    # => almost never used, should be ManyToMany(Territoire)
    date = models.DateField(
        "Date", default=datetime.date.today, blank=True, null=True
    )
    # remarques = models.TextField(blank=True)

    nom_associe = models.CharField(
        "nom associé (copie)", editable=False, max_length=255, blank=True
    )

    def save(self, force_insert=False, force_update=False):
        if self.affiliation is not None:
            if self.affiliation.structure is not None:
                self.nom_associe = self.affiliation.structure.nom
                (
                    Structure.objects.filter(
                        pk=self.affiliation.structure.id
                    ).update(adherent=True)
                )
            elif self.affiliation.personne is not None:
                self.nom_associe = "%s %s" % (
                    self.affiliation.personne.prenom,
                    self.affiliation.personne.nom,
                )
                (
                    Personne.objects.filter(
                        pk=self.affiliation.personne.id
                    ).update(adherent=True)
                )
        super(Adhesion, self).save(force_insert, force_update)

    def nom(self):
        return self.affiliation.nom()

    def prenom(self):
        return self.affiliation.prenom()

    prenom.short_description = "prénom"

    def __str__(self):
        if self.affiliation.structure is not None:
            return "Adhésion %d : %s de %s" % (
                self.pk,
                localize(self.date),
                self.affiliation.structure.nom,
            )
        elif self.affiliation.personne is not None:
            return "Adhésion %d : %s de %s %s" % (
                self.pk,
                localize(self.date),
                self.affiliation.personne.prenom,
                self.affiliation.personne.nom,
            )
        return "Adhésion %d : (orpheline)" % self.pk


class Paiement(models.Model):
    class Meta:
        abstract = True
        ordering = ["date_reglement"]

    date_facturation = models.DateField(
        "date de facturation",
        default=datetime.date.today,
        blank=True,
        null=True,
    )
    date_reglement = models.DateField(
        "date de règlement", blank=True, null=True
    )
    # TODO: (type) clean-up + choices(liste)
    type = models.CharField(max_length=255, blank=True)
    montant = models.IntegerField(blank=True, null=True)

    def __str__(self):
        return "Paiement %d : %d€ par %s le %s (règlement le %s)" % (
            self.pk,
            self.montant or 0,
            self.type,
            localize(self.date_facturation),
            localize(self.date_reglement),
        )


# 2 different tables because 2 different PKs
class ModePaiementAbonnement(Paiement):
    class Meta:
        verbose_name = "Mode de Paiement (Abonnement)"
        verbose_name_plural = "Modes de Paiement (Abonnement)"

    # Chicken-and-egg issue
    # def save(self, force_insert=False, force_update=False):
    #    self.abonnement.affiliation.update_montant_total()
    #    super(Paiement, self).save(force_insert, force_update)


class ModePaiementAdhesion(Paiement):
    class Meta:
        verbose_name = "Mode de Paiement (Adhésion)"
        verbose_name_plural = "Modes de Paiement (Adhésion)"

    # Chicken-and-egg issue
    # def save(self, force_insert=False, force_update=False):
    #    self.adhesion.affiliation.update_montant_total()
    #    super(Paiement, self).save(force_insert, force_update)


class Affiliation(models.Model):
    """
    This table is superfluous because everything can be deduced from other
    tables:
    - montant_total: computed
    - maj: max(abonnement.maj, adhesion.maj)
    - nom: copy(structure/person.nom)
    - email: id.
    - abonnement: if there's a Abonnement for the Person/Structure
    - adhesion: id.
    - ...
    """

    class Meta:
        ordering = ["personne__nom", "personne__prenom", "structure__nom"]

    personne = models.ForeignKey(
        "Personne", blank=True, null=True, on_delete=models.CASCADE
    )
    structure = models.ForeignKey(
        "Structure", blank=True, null=True, on_delete=models.CASCADE
    )
    # TODO: (etat) 2 boolean fields
    etat = models.CharField(max_length=255, blank=True)
    date_maj = models.DateField(auto_now=True)

    nom_precedent = models.CharField(
        "nom précédent (copie)", editable=False, max_length=255, blank=True
    )
    adresse_complete = models.CharField(
        max_length=255, editable=False, blank=True
    )
    email = models.EmailField(editable=False, blank=True)
    montant_total = models.IntegerField(
        "montant total (copie)", editable=False, blank=True, default=0
    )

    def save(self, force_insert=False, force_update=False):
        if self.personne is not None:
            self.nom_precedent = "%s %s" % (
                self.personne.prenom,
                self.personne.nom,
            )
            self.adresse_complete = self.personne.adresse_complete
            self.email = self.personne.email
        elif self.structure is not None:
            self.nom_precedent = "%s" % (self.structure.nom)
            # self.adresse_complete = self.structure.adresse_complete
            self.adresse_complete = self.structure.adresse_contact_etiquette
            self.email = self.structure.email
        super(Affiliation, self).save(force_insert, force_update)

    def nom(self):
        if self.structure is not None:
            return self.structure.nom
        elif self.personne is not None:
            return self.personne.nom

    def prenom(self):
        if self.personne is not None:
            return self.personne.prenom
        return "-"

    prenom.short_description = "prénom"

    masque = models.BooleanField(
        "masqué",
        default=False,
        help_text=(
            "ℹ️ Permet de masquer l’affiliation via un filtre de liste et dans "
            "certains champs. ⚠️ L’affiliation n’est plus sélectionnable dans "
            "certains outils de la base."
        ),
    )

    # Non utilisé pour le moment
    # def update_montant_total(self):
    #    montant = 0
    #    montant += self.adhesion.mode_adhesion.montant
    #    montant += self.abonnement.mode_abonnement.montant
    #    self.montant_total = montant
    #    self.save()
    def __str__(self):
        if self.personne is not None:
            return " %s %s (A. №%d)" % (
                self.personne.prenom,
                self.personne.nom,
                self.pk,
            )
        elif self.structure is not None:
            return "%s (A. №%d)" % (self.structure.nom, self.pk)
        else:
            return "Affiliation orpheline (A. №%d)" % (self.pk)


class Relation(models.Model):
    """
    Relations entre <personne> et <structure>
    """

    class Meta:
        ordering = [
            "personne__nom",
            "personne__prenom",
            "fonction",
            "structure__nom",
        ]

    personne = models.ForeignKey(
        "Personne", blank=False, null=True, on_delete=models.PROTECT
    )
    structure = models.ForeignKey(
        "Structure", blank=False, null=True, on_delete=models.PROTECT
    )
    fonction = models.ForeignKey(
        "Fonction", blank=True, null=True, on_delete=models.PROTECT
    )

    def __str__(self):
        if self.fonction is not None:
            return (
                f"{self.personne.prenom} {self.personne.nom} "
                f"est {self.fonction} de {self.structure.nom} (R. №{self.pk})"
            )
        else:
            return (
                f"{self.personne.prenom} {self.personne.nom} "
                f"est en relation avec {self.structure.nom} (R. №{self.pk})"
            )

    verbose_name_plural = "relations"


class RendezVous(models.Model):
    """
    Rendez-vous entre des référents APES et des personnes et/ou des orgas
    """

    date_debut = models.DateTimeField(
        "date et heure de début",
        blank=False,
        null=False,
        default=now,
    )
    date_fin = models.DateTimeField(
        "date et heure de fin", blank=False, null=False, default=now
    )
    personnes_apes = models.ManyToManyField(
        "Personne",
        blank=False,
        related_name="personnes_apes",
        verbose_name="personne(s) de l’APES",
        limit_choices_to={"referent_apes": True},
    )
    autres_personnes_presentes = models.ManyToManyField(
        "Personne",
        blank=False,
        related_name="autres_personnes",
        verbose_name="autres personne(s) présente(s)",
    )
    structures_presentes = models.ManyToManyField(
        "Structure", blank=True, verbose_name="structure(s) présente(s)"
    )
    commentaires = models.TextField(blank=True, null=True)
    piece_jointe = models.FileField(
        upload_to="pieces-jointes",
        blank=True,
        null=True,
    )

    class Meta:
        verbose_name_plural = "rendez-vous"
        verbose_name = "rendez-vous"

    def __str__(self):
        return f"Rendez-vous le {localize(self.date_debut)} (R. №{self.pk})"


class LienWeb(models.Model):
    personne = models.ForeignKey(
        "Personne",
        blank=True,
        null=True,
        on_delete=models.PROTECT,
        help_text=(
            "Personne à qui appartient le lien. ⚠️ Ne pas remplir en même "
            "temps que « Structure »."
        ),
    )
    structure = models.ForeignKey(
        "Structure",
        blank=True,
        help_text=(
            "Structure à qui appartient le lien. ⚠️ Ne pas remplir en même "
            "temps que « Personne »."
        ),
        null=True,
        on_delete=models.PROTECT,
    )
    url = models.URLField(
        "Lien",
        help_text=(
            "ℹ️ Copier-coller ici le lien vers la page Web en question. "
            "Débute en général par <em>https://</em> ou <em>http://</em>. "
            # "Peut être saisi sans cette partie là, avec juste le domaine "
            # "suivi de la partie de droite (ex : <em>example.org/ma/super/page"
            # "</em>)."
        ),
    )
    type_lien = models.ForeignKey(
        "TypeLien",
        blank=False,
        null=False,
        on_delete=models.PROTECT,
        verbose_name="type de lien",
        help_text=(
            "ℹ️ Site, blog, Mastodon, Facebook, Instagram, TikTok," "Twitter…"
        ),
    )

    class Meta:
        verbose_name_plural = "Liens Web"

    def __str__(self):
        return f"Lien {self.type_lien} vers {self.url} (L. №{self.pk})"


class LienAvecAPES(models.Model):
    """ """

    class Meta:
        verbose_name = "lien avec l'APES"
        verbose_name_plural = "liens avec l'APES"

    personne = models.ForeignKey(
        "Personne",
        blank=True,
        null=True,
        on_delete=models.PROTECT,
        help_text=(
            "ℹ️ Personne qui est en lien avec l’APES. ⚠️ Ne pas remplir en "
            "même temps que « Structure »."
        ),
    )
    structure = models.ForeignKey(
        "Structure",
        blank=True,
        null=True,
        on_delete=models.PROTECT,
        help_text=(
            "ℹ️ Structure qui est en lien avec l’APES. ⚠️ Ne pas remplir en"
            " même temps que « Personne »."
        ),
    )
    personne_apes = models.ForeignKey(
        "Personne",
        blank=False,
        null=False,
        on_delete=models.PROTECT,
        related_name="personne_apes",
        verbose_name="personne référente côté APES",
        limit_choices_to={
            "referent_apes": True,
            "masque": False,
        },
    )

    def __str__(self):
        if self.structure is not None:
            return (
                f"{self.structure.nom} est en lien avec l’APES via "
                f"{self.personne_apes.nom} {self.personne_apes.nom}"
                f"(L. №{self.pk})"
            )
        elif self.personne is not None:
            return (
                f"{self.personne.prenom} {self.personne.nom} est en lien"
                f" avec l’APES via {self.personne_apes.prenom} "
                f"{self.personne_apes.nom} (L. №{self.pk})"
            )
        else:
            return f"Lien avec l’APES orphelin (L. №{self.pk})"


class AmbassadeurAPES(models.Model):
    """ """

    class Meta:
        verbose_name = 'ambassadeur·ice pour l’APES'
        verbose_name_plural = 'ambassadeur·ice·s pour l’APES'

    personne = models.ForeignKey(
        "Personne",
        blank=True,
        null=True,
        on_delete=models.PROTECT,
        limit_choices_to={
            "masque": False,
        },
    )
    type_ambassadeur = models.ForeignKey(
        "TypeAmbassadeur",
        blank=True,
        null=True,
        on_delete=models.PROTECT,
    )

    def __str__(self):
        return (
            f"{self.personne.prenom} {self.personne.nom} est "
            f"{self.type_ambassadeur.libelle} (A. №{self.pk})"
        )


# Libellés
class Libelle(models.Model):
    class Meta:
        abstract = True
        ordering = ["libelle"]

    libelle = models.CharField(
        "libellé",
        help_text=(
            "ℹ️ Le nom pour les humains. ⚠️ Doît être unique et faire au "
            "maximum 255 caractères."
        ),
        max_length=255,
    )
    slug = models.SlugField(
        help_text=(
            "ℹ️ Le nom pour les machines. ⚠️ Doît être unique et faire au "
            "maximum 50 caractères."
        ),
    )

    def __str__(self):
        return self.libelle

    def _slugify(self):
        self.slug = slugify(self.libelle)


class Territoire(Libelle):
    """
    Territoires sur le(s)quel(s) une personne ou une structure est active.
    """

    parent = models.ForeignKey(
        "self",
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        related_name="enfants",
        help_text=(
            "ℹ️ Territoire plus étendu dont le territoire actuel fait"
            " entièrement partie.️ ⚠️ Un territoire ne peut pas être son"
            " propre parent ou le parent de ses parents. Comme en généalogie "
            " (sauf dans les histoires de voyage dans le temps ou de clonage "
            " 🛸)."
        ),
    )


class TypeAmbassadeur(Libelle):
    """
    Types d’ambassadeur de l'APES (personnes et structures ??)
    """

    class Meta:
        verbose_name = "type d’ambassadeur"
        verbose_name_plural = "types d’ambassadeurs"


class TypePaiement(Libelle):
    """
    Types de partenaires de l'APES (personnes et structures)
    """

    class Meta:
        verbose_name = "type de paiement"
        verbose_name_plural = "types de paiement"


class TypePartenaire(Libelle):
    """
    Types de partenaires de l'APES (personnes et structures)
    """

    class Meta:
        verbose_name = "type de partenaire"
        verbose_name_plural = "types de partenaire"


class TypeLien(Libelle):
    """
    Types des URL des personnes et structures
    """

    class Meta:
        verbose_name = "type de lien Web"
        verbose_name_plural = "types de liens Web"


# class StatutJuridique(models.Model): pass
class StatutJuridique(Libelle):
    """
    Statuts juridiques que peut avoir une structure.
    """

    class Meta:
        verbose_name = "statut juridique"
        verbose_name_plural = "statuts juridiques"


class ActiviteAPES(Libelle):
    """
    Désigne la ou les activités de l'APES auxquels des personnes et structures
    peuvent contribuer. Référencé depuis Personne et Structure.
    """

    class Meta:
        verbose_name = "activité APES"
        verbose_name_plural = "activités APES"


class DomaineActivite(Libelle):
    """
    Désigne le ou les domaines d’activités des structures peuvent appartenir.
    Référencé depuis Structure.
    """

    class Meta:
        verbose_name = "domaine d’activité"
        verbose_name_plural = "domaines d’activité"


class Fonction(Libelle):
    """
    Désigne la fonction que peut exercer une personne dans une structure.
    """

    pass


class ParticulariteStructure(Libelle):
    """
    Désigne la ou les particularités d'une structure
    """

    class Meta:
        verbose_name = "particularité d'une structures"
        verbose_name_plural = "particularités des structures"


# TypePaiement
#   espèces, chèque, virement, timbres, en attente
# Collège
#   A, B
