# -*- coding: utf-8 -*-
from django.test import TestCase

from grapes.bdd.models import (
    Abonnement,
    Adhesion,
    Affiliation,
    Personne,
    StatutJuridique,
    Structure,
)


class PageTest(TestCase):
    fixtures = ["grapes/bdd/tests/test_data"]
    credential = {"username": "admin", "password": "password"}

    @property
    def p1(self):
        return Personne(
            nom="Toto",
            prenom="Jean-Michel",
            pk=21,
            email="j-m.toto@domaine.inval",
            abonne=False,
            adherent=False,
            sol=False,
            journaliste=True,
        )

    @property
    def s1(self):
        sj1 = StatutJuridique(
            pk=1, libelle="Association loi 1901 ou assimilée"
        )
        sj1.save()
        return Structure(
            nom="Asso Machine",
            pk=13,
            email="contact@asso-machine.inval",
            statut_juridique=sj1,
            abonne=False,
            adherent=False,
            demarche_progres=False,
            demarche_progres_environnement=False,
            demarche_progres_emploi=False,
            marche_citoyen=False,
            collectifs_acteurs=True,
        )

    def test_landing_page_unauth(self):
        "❔🕸  Teste la page d’accueil."
        res = self.client.get("/", follow=True)
        self.assertEqual(res.status_code, 200)
        # on recupere l'adresse de la page apres la redirection
        login_url = res.request["PATH_INFO"]
        self.assertEqual(
            login_url,
            "/gestion/login/",
            "❌ La redirection de la page d’accueil a échoué.",
        )
        content = res.content.decode("utf-8")
        # print(content)
        assert "Base de données grAPES" in content, (
            '❌ Le texte « Base de données grAPES »'
            ' manque dans la page d’accueil.'
        )

    def test_admin_page(self):
        "❔🕸  Teste l’URL de la page d’administration."
        logged_in = self.client.login(**self.credential)
        self.assertEqual(logged_in, True)
        res = self.client.get("/", follow=True)
        content = res.content.decode("utf-8")
        assert (
            "Site d’administration" in content
        ), '❌ Le texte « Site d’administration » manque dans l’administration.'
        assert "Base de données grAPES" in content, (
            '❌ Le texte « Base de données grAPES »'
            ' manque dans l’administration.'
        )

    def test_personne_page(self):
        "❔🕸 Teste la page d’édition d’une personne."
        self.p1.save()
        self.client.login(**self.credential)
        res = self.client.get("/gestion/bdd/personne/21/change/", follow=True)
        content = res.content.decode("utf-8")
        assert (
            'type="text" name="nom" value="Toto"' in content
        ), '❌ Le champs « nom » pose problème dans la page personne.'
        assert (
            'type="text" name="prenom" value="Jean-Michel"' in content
        ), '❌ Le champs « prénom » pose problème dans la page personne.'
        assert (
            'type="email" name="email" value="j-m.toto@domaine.inval"'
        ) in content, (
            '❌ Le champs « email » pose problème dans la page personne.'
        )

    def test_affiliation_page(self):
        "❔ Teste la page d’une affiliation."
        self.p1.save()
        self.s1.save()
        Affiliation(pk=6, personne_id=21, structure_id=13)
        self.client.login(**self.credential)
        res = self.client.get(
            "/gestion/bdd/personne/21/change/affiliation/", follow=True
        )
        content = res.content.decode("utf-8")
        assert "Affiliations" in content, "site_admin"
        res = self.client.get(
            "/gestion/bdd/structure/13/change/affiliation/", follow=True
        )
        content = res.content.decode("utf-8")
        assert "Affiliations" in content, "site_admin"

    def test_statutjuridique_page(self):
        "❔ Teste la page d’un statut juridique."
        sj1 = StatutJuridique(pk=3, libelle="SCIC")
        sj1.save()
        self.client.login(**self.credential)
        res = self.client.get(
            "/gestion/bdd/statutjuridique/3/change/", follow=True
        )
        content = res.content.decode("utf-8")
        assert (
            "SCIC" in content
        ), "❌ Pas de statut SCIC listé dans le statut juridique nº3."

    def test_export_personne_mailto(self):
        "❔👤📩 Teste l’export mail d’une personne."
        self.p1.save()
        self.client.login(**self.credential)
        res = self.client.post(
            "/gestion/bdd/personne/",
            {
                "action": "generate_mailto",
                "_selected_action": "21",
            },
            follow=True,
        )
        content = res.content.decode("utf-8")
        params = [
            (
                (
                    "function go_now () { window.location.href = "
                    "'mailto:contact@apes.org, ?bcc=j-m.toto@domaine.inval'; }"
                ),
                "❌ Le JS d’envoi à des personnes manque.",
            ),
            (
                (
                    "Envoi à:<br /><a href='mailto:contact@apes.org,"
                    " ?bcc=j-m.toto@domaine.inval'>j-m.toto@domaine.inval</a>"
                ),
                "❌ Le lien HTML d’envoi à des personnes manque.",
            ),
        ]
        for search_string, error in params:
            assert search_string in content, error

    def test_export_structure_mailto(self):
        "❔🏢📩 Teste l’export mail d’une structure."
        self.s1.save()
        self.client.login(**self.credential)
        res = self.client.post(
            "/gestion/bdd/structure/",
            {
                "action": "generate_mailto",
                "_selected_action": "13",
            },
            follow=True,
        )
        content = res.content.decode("utf-8")
        params = [
            (
                (
                    "function go_now () { window.location."
                    "href = 'mailto:contact@apes.org, "
                    "?bcc=contact@asso-machine.inval'; }"
                ),
                "❌ Le JS d’envoi à des structures manque.",
            ),
            (
                (
                    "Envoi à:<br /><a href='mailto:contact@apes.org,"
                    " ?bcc=contact@asso-machine.inval'>contact@a"
                    "sso-machine.inval</a>"
                ),
                "❌ Le lien HTML d’envoi à des structures manque.",
            ),
        ]
        for search_string, error in params:
            assert search_string in content, error

    def test_export_personne_csv(self):
        "❔👤📊 Teste l’export d'une personne physique en CSV"
        self.p1.save()
        self.client.login(**self.credential)
        res = self.client.post(
            "/gestion/bdd/personne/",
            {
                "action": "export_personne_csv",
                "_selected_action": "21",
            },
            follow=True,
        )
        content = res.content.decode("utf-8")
        params = [
            (
                (
                    '"Toto";"Jean-Michel";;;;;;"j-m.toto@'
                    'domaine.inval";FAUX;FAUX;FAUX;VRAI;;;;'
                ),
                "❌ La personne n’a pu être exportée en CSV.",
            ),
        ]
        for search_string, error in params:
            assert search_string in content, error

    def test_export_structure_csv(self):
        "❔🏢📊 Teste l’export de structures avec ou sans statut juridique en CSV"
        # Test d’export d’une structure avec un statut juridique
        self.s1.save()
        self.client.login(**self.credential)
        res = self.client.post(
            "/gestion/bdd/structure/",
            {
                "action": "export_structure_csv",
                "_selected_action": "13",
            },
            follow=True,
        )
        content = res.content.decode("utf-8")
        params = [
            (
                (
                    '"Asso Machine";;;;;;;"contact@asso-machin'
                    'e.inval";;;;;"Association loi 1901 ou assimilée";;;'
                    'FAUX;FAUX;FAUX;FAUX;VRAI;FAUX;FAUX;FAUX;;;;;'
                ),
                "❌ La structure avec statut n’a pu être exportée"
                " correctement en CSV.",
            ),
        ]
        for search_string, error in params:
            assert search_string in content, error
        # Test d’export d’une structure avec un statut juridique
        s2 = Structure(
            pk=14,
            nom="Structure Ture",
            statut_juridique=None,
            abonne=False,
            demarche_progres=True,
        )
        s2.save()
        res = self.client.post(
            "/gestion/bdd/structure/",
            {
                "action": "export_structure_csv",
                "_selected_action": "14",
            },
            follow=True,
        )
        content = res.content.decode("utf-8")
        params = [
            (
                (
                    '"Structure Ture";;;;;;;;;;;;;;;VRAI;FAUX;'
                    'FAUX;FAUX;FAUX;FAUX;FAUX;FAUX;;;;;'
                ),
                "❌ La structure sans statut n’a pu être exportée en CSV.",
            ),
        ]
        for search_string, error in params:
            assert search_string in content, error

    def test_create_abonnement_personne(self):
        "❔👤📚 Teste l’abonnement d’une personne."
        self.p1.save()
        aff = Affiliation(pk=6, personne_id=21)
        aff.save()
        self.client.login(**self.credential)
        res = self.client.get(
            "/gestion/bdd/personne/21/change/affiliation/abonnement/create/",
            follow=True,
        )
        self.assertEqual(res.status_code, 200)
        url = res.request["PATH_INFO"]
        self.assertEqual(
            url, "/gestion/bdd/personne/21/change/affiliation/abonnement/1/"
        )
        params = [
            (
                "<h1>Abonnement</h1>",
                '❌ Le titre « Abonnement" manque au formulaire.',
            ),
            (
                'name="nb_exemplaires"',
                '❌ Le champs « nb_exemplaires » manque au formulaire.',
            ),
            (
                'type="submit" value="Enregistrer"',
                "❌ Le bouton d’envoi du formulaire manque.",
            ),
        ]
        for search_string, error in params:
            assert search_string in str(res.content), error
        res = self.client.post(
            "/gestion/bdd/personne/21/change/affiliation/abonnement/1/",
            {
                "nb_exemplaires": "3",
                "date": "2022-11-06",
                "initial-date": "2022-11-06",
            },
        )
        res = self.client.get(
            "/gestion/bdd/personne/21/change/affiliation/abonnement/1/"
        )
        params = [
            (
                'type="number" name="nb_exemplaires" value="3"',
                '❌ Le champs « nb_exemplaires » n’a pas été modifié.',
            ),
            (
                'type="text" name="date" value="06/11/2022"',
                '❌ Le champs « date » n’a pas été modifié.',
            ),
            (
                'type="hidden" name="initial-date" value="2022-11-06"',
                '❌ Le champs « initial-date » n’a pas été modifié.',
            ),
        ]
        for search_string, error in params:
            assert search_string in str(res.content), error

    def test_create_adhesion_personne(self):
        "❔👤🎫 Teste l’adhésion d’une personne."
        self.p1.save()
        aff1 = Affiliation(pk=6, personne_id=21)
        aff1.save()
        self.client.login(**self.credential)
        res = self.client.get(
            (
                "/gestion/bdd/personne/21/change/"
                "affiliation/adhesion/create/"
            ),
            follow=True,
        )
        self.assertEqual(res.status_code, 200)
        url = res.request["PATH_INFO"]
        self.assertEqual(
            url, "/gestion/bdd/personne/21/change/affiliation/adhesion/1/"
        )
        content = res.content.decode("utf-8")
        params = [
            ('<h1>Adhésion ', '❌ Le titre « Adhésion » manque au formulaire.'),
            (
                "<p>Collège : <b>A",
                '❌ L’information « college » manque au formulaire.',
            ),
            (
                'type="submit" value="Enregistrer"',
                "❌ Le bouton d’envoi du formulaire manque.",
            ),
        ]
        for search_string, error in params:
            assert search_string in content, error
        res = self.client.post(
            "/gestion/bdd/personne/21/change/affiliation/adhesion/1/",
            {
                #            'college' : 'A',
                "date": "2022-10-06",
                "initial-date": "2022-10-06",
            },
        )
        res = self.client.get(
            "/gestion/bdd/personne/21/change/affiliation/adhesion/1/"
        )
        content = res.content.decode("utf-8")
        params = [
            (u"<p>Collège : <b>A", "❌ Le collège n’est pas présent."),
            (
                'type="text" name="date" value="06/10/2022"',
                '❌ Le champs « date » n’a pas été modifié.',
            ),
            (
                'type="hidden" name="initial-date" value="2022-10-06"',
                '❌ Le champs « initial-date » n’a pas été modifié.',
            ),
        ]
        for search_string, error in params:
            assert search_string in content, error

    def test_create_adhesion_structure(self):
        "❔👤🎫 Teste l’adhésion d’une structure."
        self.s1.save()
        aff = Affiliation(pk=6, structure_id=13)
        aff.save()
        self.client.login(**self.credential)
        res = self.client.get(
            (
                "/gestion/bdd/structure/13/change/"
                "affiliation/adhesion/create/"
            ),
            follow=True,
        )
        self.assertEqual(res.status_code, 200)
        url = res.request["PATH_INFO"]
        self.assertEqual(
            url, "/gestion/bdd/structure/13/change/affiliation/adhesion/1/"
        )
        content = res.content.decode("utf-8")
        params = [
            ('<h1>Adhésion', '❌ Le titre « Adhésion » manque au formulaire.'),
            ('name="college"', '❌ Le menu « college » manque au formulaire.'),
            (
                'type="submit" value="Enregistrer"',
                "❌ Le bouton d’envoi du formulaire manque.",
            ),
        ]
        for search_string, error in params:
            assert search_string in content, error
        res = self.client.post(
            "/gestion/bdd/structure/13/change/affiliation/adhesion/1/",
            {
                "college": "B",
                "date": "2022-10-06",
                "initial-date": "2022-10-06",
            },
            follow=True,
        )
        res = self.client.get(
            ("/gestion/bdd/structure/13/change/" "affiliation/adhesion/1/")
        )
        content = res.content.decode("utf-8")
        params = [
            (
                'option value="B" selected>',
                "❌ Le collège n’a pas été modifié.",
            ),
            (
                'type="text" name="date" value="06/10/2022"',
                '❌ Le champs « date » n’a pas été modifié.',
            ),
            (
                'type="hidden" name="initial-date" value="2022-10-06"',
                '❌ Le champs « initial-date » n’a pas été modifié.',
            ),
        ]
        for search_string, error in params:
            assert search_string in content, error

    def test_create_paiement_abonnement(self):
        "❔💶 Teste le paiement d’un abonnement."
        self.s1.save()
        aff = Affiliation(pk=6, structure_id=13)
        aff.save()
        abo = Abonnement(affiliation_id=6, date="2022-10-08")
        abo.save()
        self.client.login(**self.credential)
        res = self.client.get(
            (
                "/gestion/bdd/structure/13/change/"
                "affiliation/abonnement/paiement/create/"
            ),
            follow=True,
        )
        url = res.request["PATH_INFO"]
        self.assertEqual(
            url,
            "/gestion/bdd/structure/13/change/"
            "affiliation/abonnement/paiement/1/",
        )
        content = res.content.decode("utf-8")
        params = [
            (
                "<h1>Paiement abonnement</h1>",
                "❌ Il n’a pas de titre au formulaire de "
                "paiement d’abonnement.",
            ),
            (
                ' type="text" name="date_facturation"',
                '❌ Le champs « date_facturation » n’existe pas',
            ),
            (
                ' type="text" name="date_reglement"',
                '❌ Le champs « date_reglement » n’existe pas.',
            ),
            (' type="text" name="type"', '❌ Le champs « type » n’existe pas.'),
            (
                ' type="number" name="montant"',
                '❌ Le champs « montant » n’existe pas.',
            ),
        ]
        for search_string, error in params:
            assert search_string in content, error
        res = self.client.post(
            "/gestion/bdd/structure/13/change/"
            "affiliation/abonnement/paiement/1/",
            {
                "date_facturation": "2021-05-01",
                "date_reglement": "2021-05-06",
                "type": "CB",
                "montant": "100",
            },
            follow=True,
        )
        url = res.request["PATH_INFO"]
        self.assertEqual(url, "/gestion/bdd/structure/13/change/affiliation/")
        params = [
            (
                "<h1>Paiement abonnement</h1>",
                "❌ Il n’a pas de titre au formulaire de paiement d’abonnement.",
            ),
            (
                '<input type="text" name="date_facturation"',
                '❌ Le champs « date_facturation » n’existe pas',
            ),
            (
                '<input type="text" name="date_reglement"',
                '❌ Le champs « date_reglement » n’existe pas.',
            ),
            ('type="text" name="type"', '❌ Le champs « type » n’existe pas.'),
            (
                '<input type="text" name="montant"',
                '❌ Le champs « montant » n’existe pas.',
            ),
        ]
        res = self.client.get(
            (
                "/gestion/bdd/structure/13/change/"
                "affiliation/abonnement/paiement/1/"
            ),
            follow=True,
        )
        params = [
            (
                {
                    '<input type="text" name="date_facturation"'
                    ' value="2022-05-01"'
                },
                '❌ Le champs « date_facturation » n’a pas été modifié',
            ),
            (
                '<input type="text" name="date_reglement" value="2022-05-06"',
                '❌ Le champs « date_reglement » n’a pas été modifié.',
            ),
            (
                'type="text" name="type" value="CB"',
                '❌ Le champs « type » n’a pas été modifié.',
            ),
            (
                '<input type="text" name="montant" value="100"',
                '❌ Le champs « montant » n’a pas été modifié.',
            ),
        ]

    def test_create_paiement_adhesion(self):
        "❔💶 Teste le paiement d’une adhésion."
        self.s1.save()
        aff = Affiliation(pk=6, structure_id=13)
        aff.save()
        adh = Adhesion(affiliation_id=6, college="B", date="2022-10-08")
        adh.save()
        self.client.login(**self.credential)
        res = self.client.get(
            "/gestion/bdd/structure/13/change/"
            "affiliation/adhesion/paiement/create/",
            follow=True,
        )
        url = res.request["PATH_INFO"]
        self.assertEqual(
            url,
            "/gestion/bdd/structure/13/change/"
            "affiliation/adhesion/paiement/1/",
        )
        content = res.content.decode("utf-8")
        params = [
            (
                '<h1>Paiement adhésion</h1>',
                '❌ Il n’a pas de titre au formulaire de paiement d’adhesion.',
            ),
            (
                ' type="text" name="date_facturation"',
                '❌ Le champs « date_facturation » n’existe pas'
                ' ou ne s’affiche pas correctement.',
            ),
            (
                'type="text" name="date_reglement"',
                '❌ Le champs « date_reglement » n’existe pas'
                ' ou ne s’affiche pas correctement.',
            ),
            (
                'type="text" name="type"',
                '❌ Le champs « type » n’existe pas ou ne'
                ' s’affiche pas correctement.',
            ),
            ('type="text" name="type"', 'Le champs "type" n’existe pas.'),
            (
                'type="number" name="montant"',
                '❌ Le champs « montant » n’existe pas'
                ' ou ne s’affiche pas correctement.',
            ),
        ]
        for search_string, error in params:
            assert search_string in content, error
        res = self.client.post(
            "/gestion/bdd/structure/13/change/affiliation/adhesion/paiement/1/",
            {
                "date_facturation": "2021-05-01",
                "date_reglement": "2021-05-06",
                "type": "CB",
                "montant": "100",
            },
            follow=True,
        )
        url = res.request["PATH_INFO"]
        self.assertEqual(url, "/gestion/bdd/structure/13/change/affiliation/")
        res = self.client.get(
            (
                "/gestion/bdd/structure/13/change/"
                "affiliation/adhesion/paiement/1/"
            ),
            follow=True,
        )
        content = res.content.decode("utf-8")
        params = [
            (
                ('type="text" name="date_facturation"' ' value="01/05/2021"'),
                '❌ Le champs « date_facturation » n’a pas été modifié.',
            ),
            (
                'type="text" name="date_reglement" value="06/05/2021"',
                '❌ Le champs « date_reglement » n’a pas été modifié.',
            ),
            (
                'type="text" name="type" value="CB"',
                '❌ Le champs « type » n’a pas été modifié.',
            ),
            (
                ' type="number" name="montant" value="100"',
                '❌ Le champs « montant » n’a pas été modifié.',
            ),
        ]
        for search_string, error in params:
            assert search_string in content, error
