# -*- coding: utf-8 -*-

from datetime import date, timedelta

from django.test import TestCase
from django.utils.formats import localize

from grapes.bdd.models import (
    Abonnement,
    Adhesion,
    Affiliation,
    ModePaiementAbonnement,
    ModePaiementAdhesion,
    Personne,
    Structure,
)


class ModelStrTest(TestCase):
    @property
    def p1(self):
        return Personne(
            nom="Toto",
            prenom="Jean-Michel",
            pk=21,
            abonne=False,
            journaliste=False,
            sol=False,
            adherent=False,
        )

    @property
    def s1(self):
        return Structure(
            nom="Asso Machine",
            pk=13,
            abonne=False,
            adherent=False,
            collectifs_acteurs=False,
        )

    @property
    def aff1(self):
        return Affiliation(personne_id=21)

    def test_personne(self):
        self.assertEqual(str(self.p1), "Jean-Michel Toto (P. №21)")

    def test_nomcomplet_personne(self):
        self.assertEqual(str(self.p1.nom_complet()), "Jean-Michel Toto")

    def test_structure(self):
        self.assertEqual(str(self.s1), "Asso Machine (S. №13)")

    def test_nomcomplet_structure(self):
        self.assertEqual(str(self.s1.nom_complet()), "Asso Machine")

    def test_affiliations(self):
        self.p1.save()
        self.s1.save()
        params = [
            (None, None, "Affiliation 6 : (orpheline)", None, "-"),
            (None, 13, "Affiliation 6 : Asso Machine", "Asso Machine", "-"),
            (
                21,
                None,
                "Affiliation 6 : Jean-Michel Toto",
                "Toto",
                "Jean-Michel",
            ),
            (
                21,
                13,
                "Affiliation 6 : Jean-Michel Toto",
                "Asso Machine",
                "Jean-Michel",
            ),
        ]
        for pers_id, struct_id, exp_value, exp_nom, exp_prenom in params:
            aff = Affiliation(
                pk=6, personne_id=pers_id, structure_id=struct_id
            )
            aff.save()
            self.assertEqual(str(aff), exp_value)
            self.assertEqual(aff.nom(), exp_nom)
            self.assertEqual(aff.prenom(), exp_prenom)

    def test_abonnement(self):
        self.p1.save()
        self.s1.save()
        params = [
            (None, None, "Abonnement 7 : (orphelin)", None, "-"),
            (
                None,
                13,
                "Abonnement 7 : 9 novembre 2022 de Asso Machine",
                "Asso Machine",
                "-",
            ),
            (
                21,
                None,
                "Abonnement 7 : 9 novembre 2022 de Jean-Michel Toto",
                "Toto",
                "Jean-Michel",
            ),
            (
                21,
                13,
                "Abonnement 7 : 9 novembre 2022 de Asso Machine",
                "Asso Machine",
                "Jean-Michel",
            ),
        ]
        for pers_id, struct_id, exp_value, exp_nom, exp_prenom in params:
            aff = Affiliation(
                pk=6, personne_id=pers_id, structure_id=struct_id
            )
            aff.save()
            abo = Abonnement(pk=7, affiliation_id=6, date=date(2022, 11, 9))
            abo.save()
            self.assertEqual(str(abo), exp_value)
            self.assertEqual(abo.nom(), exp_nom)
            self.assertEqual(abo.prenom(), exp_prenom)

    #            try:
    #                Abonnement(
    #                    pk=8,
    #                    affiliation=None,
    #                    date=date(2022, 11, 9)
    #                ).save()
    #            except:
    #                self.assertRaises("ValueError")

    def test_adhesion(self):
        self.p1.save()
        self.s1.save()
        params = [
            (None, None, "Adhésion 8 : (orpheline)", None, "-"),
            (
                None,
                13,
                "Adhésion 8 : 9 novembre 2022 de Asso Machine",
                "Asso Machine",
                "-",
            ),
            (
                21,
                None,
                "Adhésion 8 : 9 novembre 2022 de Jean-Michel Toto",
                "Toto",
                "Jean-Michel",
            ),
            (
                21,
                13,
                "Adhésion 8 : 9 novembre 2022 de Asso Machine",
                "Asso Machine",
                "Jean-Michel",
            ),
        ]
        for pers_id, struct_id, exp_value, exp_nom, exp_prenom in params:
            aff = Affiliation(
                pk=6, personne_id=pers_id, structure_id=struct_id
            )
            aff.save()
            adh = Adhesion(
                pk=8, affiliation_id=6, date=date(2022, 11, 9), college="A"
            )
            adh.save()
            self.assertEqual(str(adh), exp_value)
            self.assertEqual(adh.nom(), exp_nom)
            self.assertEqual(adh.prenom(), exp_prenom)

    def test_modepaiementabonnement(self):
        modp = ModePaiementAbonnement(
            date_facturation=date(2022, 10, 9),
            date_reglement=date(2022, 11, 9),
            type="cheque",
            montant=15,
            pk=1,
        )
        self.assertEqual(
            str(modp),
            (
                "Paiement 1 : 15€ par cheque le 9 octobre 2022"
                " (règlement le 9 novembre 2022)"
            ),
        )

    def test_getcollege(self):
        self.p1.save()
        self.s1.save()
        # Teste la détermination d’un collège sans affiliation ni adhésion
        self.assertEqual(self.p1.get_college(), "")
        aff1 = Affiliation(pk=6, personne_id=21, structure_id=13)
        aff1.save()
        # Teste la détermination d’un collège sans adhésion
        self.assertEqual(self.p1.get_college(), "")
        adh1 = Adhesion(
            pk=8, affiliation_id=6, date=date(2022, 0o1, 11), college="A"
        )
        adh1.save()
        # Teste la détermination d’un collège avec affiliation et adhésion
        self.assertEqual(str(self.p1.get_college()), "A")
        self.assertEqual(str(self.s1.get_college()), "A")

    def test_modepaiementadhesion(self):
        modpad = ModePaiementAdhesion(
            date_facturation=date(2022, 11, 10),
            date_reglement=date(2022, 11, 15),
            type="cheque",
            montant=20,
            pk=1,
        )
        modpad.save()
        self.assertEqual(
            str(modpad),
            (
                "Paiement 1 : 20€ par cheque le 10 novembre 2022 "
                "(règlement le 15 novembre 2022)"
            ),
        )

    def test_echeanceabonnement(self):
        "❔📅📚 Teste l’échéance d’un abonnement"
        self.p1.save()
        self.s1.save()
        aff = Affiliation(pk=112, personne_id=21, structure_id=13)
        aff.save()
        # Teste une échéance d’abonnement pour un abonnement jamais facturé
        self.assertEqual(self.p1.echeance_abonnement(), None)
        abo1 = Abonnement(pk=2, affiliation_id=112, date=date(2013, 0o5, 0o1))
        # Teste une échéance d’abonnement pour un abonnement facturé il y’a plus
        # d’un an
        abo1.save()
        self.assertEqual(
            str(self.p1.echeance_abonnement()), "<em>1 mai 2013</em>"
        )
        self.assertEqual(
            str(self.p1.echeance_abonnement(htmlized=False)), "1 mai 2013"
        )
        # Teste une échéance d’abonnement pour un abonnement facturé il y’a
        # moins d’un an
        abo1 = Abonnement(
            pk=2, affiliation_id=112, date=date.today() - timedelta(200)
        )
        abo1.save()
        self.assertEqual(
            self.p1.echeance_abonnement(),
            localize(date.today() - timedelta(200)),
        )

    def test_echeanceadhesion(self):
        "❔📅🎫 Teste l’échéance d’une adhésion"
        self.p1.save()
        self.s1.save()
        aff = Affiliation(pk=112, personne_id=21, structure_id=13)
        aff.save()
        # Teste une échéance d’adhésion pour une adhésion jamais facturé
        self.assertEqual(self.p1.echeance_adhesion(), None)
        adh = Adhesion(pk=2, affiliation_id=112, date=date(2013, 0o5, 0o1))
        adh.save()
        # Teste une échéance d’adhésion pour une adhésion facturée il y’a plus
        # d’un an
        self.assertEqual(
            str(self.p1.echeance_adhesion()), "<em>1 mai 2013</em>"
        )
        self.assertEqual(
            str(self.p1.echeance_adhesion(htmlized=False)), "1 mai 2013"
        )
        # Teste une échéance d’adhésion pour une adhésion facturée il y’a moins
        # d’un an
        adh = Adhesion(
            pk=2, affiliation_id=112, date=date.today() - timedelta(200)
        )
        adh.save()
        self.assertEqual(
            self.p1.echeance_adhesion(),
            localize(date.today() - timedelta(200)),
        )
